/****************************************************
PureRa (PureRarc.rc) (c) James "jpshortstuff" Price and Fred de Vries, 2008-9

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
************************************************************/

#include "PureRa.h"

const WORD DAYS[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

/* Adds a string to the end of a StringList. */
StringList *addString(StringList *p, wchar *string){	
	/* Create the new node. */
	StringList *temp = (StringList*)(calloc(1, sizeof(StringList)));
	temp->string = (wchar*)(calloc(wcslen(string)+1, sizeof(wchar)));
	wcscpy(temp->string, string);
	temp->next = NULL;
	
	if (p == NULL)
		return temp;
	
	StringList *loop = p;
	
	/* Append the existing list to the new node. */
	while (loop->next != NULL){
		loop = loop->next;
	}
	
	loop->next = temp;
	
	return p;
}

/* Recursively free list structure. */
void freeStringList(StringList *p){
	if (p == NULL)
		return;
	
	if (p->next != NULL)
		freeStringList(p->next);
		
	free(p->string);
	free(p);
}

/* Sort a StringList using bubble sort. */
StringList *sortStringList(StringList *list){	
	StringList *current;
	StringList *last;
	StringList *end = NULL;
	StringList *temp;
	int swaps = 1;
	
	while (swaps != 0 && end != list){
		swaps = 0;
		current = list;
		last = NULL;
		
		while (current->next != end){		
			if (wcscmpi(current->string, current->next->string) > 0){				
				if (last == NULL){
					temp = current->next->next;
					list = current->next;
					list->next = current;
					list->next->next = temp;
					
					last = list;
				}
				else {
					temp = current->next->next;
					last->next = current->next;
					last->next->next = current;
					last->next->next->next = temp;
					
					last = last->next;
				}
				
				swaps = 1;
			}
			else {
				last = current;			
				current = current->next;
			}
		}
		end = current;
	}
	
	return list;
}

/* Get list of files from the given directory, or folders if 'dirs' is TRUE. */
StringList *getFiles(wchar *dir, wchar *filter, int tDays, BOOL dirs){
    wchar *search = (wchar*)calloc(wcslen(dir) + wcslen(filter) + 2, sizeof(wchar));
	wsprintf(search, L"%s\\%s", dir, filter);

    SYSTEMTIME *tFilter = NULL;
    if (tDays != 0){
        tFilter = (SYSTEMTIME*)calloc(1, sizeof(SYSTEMTIME));
        
        GetLocalTime(tFilter);
        while (tDays > tFilter->wDay){
            tFilter->wMonth--;
    		tDays -= tFilter->wDay;
    		tFilter->wDay = DAYS[tFilter->wMonth -1];
    																				
    		if (tFilter->wMonth==0){
    			tFilter->wYear--;
    			tFilter->wMonth = 12;
    		}
    	}
    	tFilter->wDay -= (WORD)tDays ;
    }

	StringList *files = NULL;	
	WIN32_FIND_DATA file;
		
	HANDLE sHandle = FindFirstFile(search, &file);

	if (sHandle == INVALID_HANDLE_VALUE){
		return NULL;
	}
	
	do {
		if ((dirs?file.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY:!(file.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) && wcscmp(file.cFileName,L".") != 0 && wcscmp(file.cFileName, L"..") != 0){
            
            SYSTEMTIME cTime, mTime;
    		FileTimeToSystemTime(&(file.ftCreationTime), &cTime);
    		FileTimeToSystemTime(&(file.ftLastWriteTime), &mTime);
    		
    		if (tFilter != NULL){
    			int cvalid = 1, mvalid = 1;
    								
    			if (cTime.wYear<(*tFilter).wYear)
    				cvalid = 0;
    			else if (cTime.wYear==(*tFilter).wYear && cTime.wMonth<(*tFilter).wMonth)
    				cvalid = 0;
    			else if (cTime.wYear==(*tFilter).wYear && cTime.wMonth==(*tFilter).wMonth && cTime.wDay<(*tFilter).wDay)
    				cvalid = 0;
    				
    			if (mTime.wYear<(*tFilter).wYear)
    				mvalid = 0;
    			else if (mTime.wYear==(*tFilter).wYear && mTime.wMonth<(*tFilter).wMonth)
    				mvalid = 0;
    			else if (mTime.wYear==(*tFilter).wYear && mTime.wMonth==(*tFilter).wMonth && mTime.wDay<(*tFilter).wDay)
    				mvalid = 0;
    			
    			if (cvalid||mvalid)
    				continue;
    		}
            
			files = addString(files, file.cFileName);
		}
	}
	while (FindNextFile(sHandle, &file)!=0);
	
	FindClose(sHandle);
	
	return files;
}
