/****************************************************
PureRa (PureRarc.rc) (c) James "jpshortstuff" Price and Fred de Vries, 2008-9

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
************************************************************/

#include "PureRa.h"

HINSTANCE hInst;

HWND hDiag; /* Main dialog */

HWND first, second; /* Screens. */

FILE *logfile = NULL; /* Log file */

int bytes; /* Bytes cleaned */

/* Expand all the environment variables in a given string. */
wchar *expand(wchar *str){
    DWORD sz = ExpandEnvironmentStrings(str, NULL, 0) + 1;
    wchar *buf = (wchar*)calloc(sz, sizeof(wchar));
    ExpandEnvironmentStrings(str, buf, sz);
    return buf;
}

/* Print a string to the log and flush output buffer. */
void logit(LPTSTR buf){
    if (logfile == NULL)
        return;
    
	fwrite(buf, sizeof(wchar), wcslen(buf), logfile);
	fflush(logfile);
}

/* Construct log path. */
wchar *makeLogPath(){
    wchar *logpath = expand(L"%systemdrive%\\PureRa.txt");
	return logpath;
}

/* Create PureRa.txt log in %systemdrive%. Return FALSE on failure. */
BOOL createLog(){
    wchar *logpath = makeLogPath();   
	
	/* Open log. */
    logfile = _wfopen(logpath, L"wb");	
	free(logpath);
	
	if (logfile == NULL)
		return FALSE;
	
	/* Print version to log. */
	logit(L"RaProducts' PureRa v");
    logit((wchar*)VERSION);
    logit(L"\r\n");

	/* Print current time to log. */
	SYSTEMTIME time;
	GetLocalTime(&time);
	wchar *temp = (wchar*)calloc(75, sizeof(wchar));
	wsprintf(temp, L"Log created at %02u:%02u on %02u/%02u/%u ", time.wHour, time.wMinute, time.wDay, time.wMonth, time.wYear);
	logit(temp);
	
	/* Print username to log. */
	ExpandEnvironmentStrings(L"(%username%)\r\n", temp, MAX_PATH);
	logit(temp);	
	free(temp);
		
	return TRUE;
}

/* Print EOF, close *log and open log in notepad. */
void finishLog(){
    if (logfile == NULL)
        return;
    
    logit(L"\r\n\r\n-=E.O.F=-");    
    fclose(logfile);
    logfile = NULL;
    
    wchar *logpath = makeLogPath();
    wchar *notepad = expand(L"%systemroot%\\system32\\notepad.exe");
    
    wchar *cmd = (wchar*)calloc(37 + wcslen(logpath), sizeof(wchar));
    wsprintf(cmd, L"%s \"%s\"", notepad, logpath);
    free(logpath);
    
    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    ZeroMemory(&si, sizeof(si));
    si.cb = sizeof(si);
    ZeroMemory(&pi, sizeof(pi));                   
    CreateProcess(notepad, cmd, NULL, NULL, 0, 0, NULL, NULL, &si, &pi);
    free(cmd);
    free(notepad);
}

void printLastError(){
    wchar *buf;
    FormatMessage(            
        FORMAT_MESSAGE_ALLOCATE_BUFFER | 
        FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS |
        FORMAT_MESSAGE_MAX_WIDTH_MASK,
        NULL,
        GetLastError(),
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (void*)&buf,
        0, NULL
    );
    logit(buf);
    LocalFree(buf);
}

/* Update status text. */
void updateStatus(wchar *text){
    SetDlgItemText(hDiag, IDC_STATUS, text);
}

/* Delete file. */
BOOL deleteFile(wchar *file, BOOL log){
    if (log){
        logit(L"\r\n");
        logit(file);
        logit(L" <- ");
    }
    
    DWORD sz;
    WIN32_FILE_ATTRIBUTE_DATA data;
    if (GetFileAttributesEx(file, GetFileExInfoStandard, &data) != 0)
        sz = data.nFileSizeLow;

    BOOL ret = DeleteFile(file);

    if (ret){
        bytes += sz;
        if (log){
            logit(L"Successfully deleted.");
        }
    }
    else if (log)
        printLastError();
       
    return ret;
}

void emptyFolder(wchar *dir){
    StringList *temp;
    
    /* Obtain list of files and delete them. */
    StringList *files = getFiles(dir, L"*", 0, 0);
    temp = files;
    while (temp != NULL){
        wchar *path = (wchar*)calloc(wcslen(dir) + wcslen(temp->string) + 2, sizeof(wchar));
        wsprintf(path, L"%s\\%s", dir, temp->string);
        deleteFile(path, 0);
        free(path);
        temp = temp->next;
    }
    freeStringList(files);
    
    /* Obtain list of folders, then empty and delete them. */
    StringList *folders = getFiles(dir, L"*", 0, 1);
    temp = folders;
    while (temp != NULL){
        wchar *path = (wchar*)calloc(wcslen(dir) + wcslen(temp->string) + 2, sizeof(wchar));
        wsprintf(path, L"%s\\%s", dir, temp->string);
        emptyFolder(path);
        RemoveDirectory(path);
        free(path);
        temp = temp->next;
    }
    freeStringList(folders);
}

/* Recurisvely search for files matching the given filters. */
void delSearch(wchar *dir, StringList *filters){
    StringList *tempFilter, *temp;
    
    /* Look for files matching filters. */
    tempFilter = filters;
    while (tempFilter != NULL){
        StringList *files = getFiles(dir, tempFilter->string, 0, 0);
        temp = files;
        while (temp != NULL){
            wchar *path = (wchar*)calloc(wcslen(dir) + wcslen(temp->string) + 2, sizeof(wchar));
            wsprintf(path, L"%s\\%s", dir, temp->string);
            deleteFile(path, 1);
            free(path);
            temp = temp->next;
        }
        freeStringList(files);
        
        tempFilter = tempFilter->next;
    }
    
    /* Obtain list of folders, then search them. */
    StringList *folders = getFiles(dir, L"*", 0, 1);
    temp = folders;
    while (temp != NULL){
        wchar *path = (wchar*)calloc(wcslen(dir) + wcslen(temp->string) + 2, sizeof(wchar));
        wsprintf(path, L"%s\\%s", dir, temp->string);
        delSearch(path, filters);
        free(path);
        temp = temp->next;
    }
    freeStringList(folders);
}

/* Cleaning function */
void clean(HWND hDlg){
    bytes = 0;
    
    /* Create log if needed */
    if (IsDlgButtonChecked(hDlg, IDC_CREATELOG) == BST_CHECKED){
        updateStatus(L"Creating log file...");
        if (!createLog()){
            DWORD ret = MessageBox(hDiag, L"Unable to create log file. Continue anyway?", L"PureRa - Log Error", MB_ICONERROR | MB_YESNO);
            if (ret == IDNO){
                updateStatus(L"Cleaning cancelled.");
                return;
            }
        }
    }
    
    StringList *fileFilters = NULL;
    
    /* ********** *.SQM ********** */
    if (IsDlgButtonChecked(hDlg, IDC_SQM) == BST_CHECKED)
        fileFilters = addString(fileFilters, L"*.SQM");

    /* ********** Config.MSI ********** */
    if (IsDlgButtonChecked(hDlg, IDC_CONFIGMSI) == BST_CHECKED){
        updateStatus(L"Emptying Config.MSI...");
        
        wchar *confmsi = expand(L"%systemdrive%\\Config.MSI");
        emptyFolder(confmsi);
        
        logit(L"\r\n");
        logit(confmsi);
        logit(L" emptied.");
        
        free(confmsi);
    }

    /* ********** FNTCACHE ********** */
    if (IsDlgButtonChecked(hDlg, IDC_FNTCACHE) == BST_CHECKED){
        updateStatus(L"Deleting FNTCACHE.DAT...");
        
        wchar *fntcache = expand(L"%systemroot%\\system32\\FNTCACHE.DAT");
        deleteFile(fntcache, 1);
        free(fntcache);
    }
     
    /* ********** IconCache ********** */
    if (IsDlgButtonChecked(hDlg, IDC_ICONCACHE) == BST_CHECKED)
        fileFilters = addString(fileFilters, L"*IconCache.db");
        
    /* ********** Recycle Bin ********** */
    if (IsDlgButtonChecked(hDlg, IDC_RECYCLEBIN) == BST_CHECKED){
        updateStatus(L"Emptying recycle bin...");
        
        SHEmptyRecycleBin(NULL, NULL, SHERB_NOCONFIRMATION | SHERB_NOPROGRESSUI | SHERB_NOSOUND);
        logit(L"\r\nRecycle bin emptied.");
    }
    
    /* ********** Spybot S&D ********** */
    if (IsDlgButtonChecked(hDlg, IDC_RECYCLEBIN) == BST_CHECKED){
        updateStatus(L"Cleaning Spybot S&D...");
        
        HKEY key;
        if (RegOpenKeyEx(HKEY_CURRENT_USER, L"Software\\Safer Networking Limited\\SpybotSnD", 0, KEY_QUERY_VALUE, &key) == ERROR_SUCCESS){
            DWORD sz;
            RegQueryValueEx(key, L"Path", NULL, NULL, NULL, &sz);
            
            wchar *buf = (wchar*)calloc(sz + 8, sizeof(wchar));
            if (RegQueryValueEx(key, L"Path", NULL, NULL, (BYTE*)buf, &sz) == ERROR_SUCCESS){
                wcscat(buf, L"Updates");
                
                StringList *zips = getFiles(buf, L"*.zip", 0, FALSE);
                StringList *temp = zips;
                while (temp != NULL){
                    wchar *path = (wchar*)calloc(wcslen(buf) + wcslen(temp->string) + 2, sizeof(wchar));
                    wsprintf(path, L"%s\\%s", buf, temp->string);
                    deleteFile(path, 1);
                    free(path);
                    temp = temp->next;
                }
                freeStringList(zips);
                free(buf);
                
                HKEY shkey;
                if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", 0, KEY_QUERY_VALUE, &shkey) == ERROR_SUCCESS){
                    RegQueryValueEx(shkey, L"Common AppData", NULL, NULL, NULL, &sz);
                    
                    buf = (wchar*)calloc(sz + 32, sizeof(wchar));
                    if (RegQueryValueEx(shkey, L"Common AppData", NULL, NULL, (BYTE*)buf, &sz) == ERROR_SUCCESS){
                        wcscat(buf, L"\\Spybot - Search & Destroy\\Logs");
                        
                        emptyFolder(buf);
                        logit(L"\r\n");
                        logit(buf);
                        logit(L" emptied.");
                    }
                    RegCloseKey(shkey);
                }
            }
            RegCloseKey(key);
        }
    }
    
    /* ********** Software Distribution ********** */
    if (IsDlgButtonChecked(hDlg, IDC_SOFTDIST) == BST_CHECKED){
        updateStatus(L"Cleaning Software Distribution...");
        
        wchar *dslogs = expand(L"%systemroot%\\SoftwareDistribution\\DataStore\\Logs");
        emptyFolder(dslogs);        
        logit(L"\r\n");
        logit(dslogs);
        logit(L" emptied.");
        free(dslogs);
        
        wchar *dl = expand(L"%systemroot%\\SoftwareDistribution\\Download");
        emptyFolder(dl);        
        logit(L"\r\n");
        logit(dl);
        logit(L" emptied.");
        free(dl);
        
        wchar *suDef = expand(L"%systemroot%\\SoftwareDistribution\\SelfUpdate\\Default");
        emptyFolder(suDef);        
        logit(L"\r\n");
        logit(suDef);
        logit(L" emptied.");
        free(suDef);
        
        wchar *wuredir = expand(L"%systemroot%\\SoftwareDistribution\\WuRedir");
        emptyFolder(wuredir);        
        logit(L"\r\n");
        logit(wuredir);
        logit(L" emptied.");
        free(wuredir);
        
        wchar *relog = expand(L"%systemroot%\\SoftwareDistribution\\ReportingEvents.log");
        deleteFile(relog, 1);
        free(relog);
    }
    
    /* ********** Temp ********** */
    if (IsDlgButtonChecked(hDlg, IDC_TEMP) == BST_CHECKED){
        updateStatus(L"Emptying Temp folder...");
        
        wchar *usrtemp = expand(L"%TEMP%");
        emptyFolder(usrtemp);        
        logit(L"\r\n");
        logit(usrtemp);
        logit(L" emptied.");
        free(usrtemp);
        
        wchar *wintemp = expand(L"%systemroot%\\TEMP");
        emptyFolder(wintemp);        
        logit(L"\r\n");
        logit(wintemp);
        logit(L" emptied.");
        free(wintemp);
    }
    
    /* ********** Thumbnail Caches ********** */
    if (IsDlgButtonChecked(hDlg, IDC_THUMBSDB) == BST_CHECKED){
        fileFilters = addString(fileFilters, L"THUMBS.DB");
        fileFilters = addString(fileFilters, L"EHTHUMBS.DB");
        fileFilters = addString(fileFilters, L"THUMBNAIL.DB");
        fileFilters = addString(fileFilters, L"THUMBCACHE_*.DB");
    }
    
    /* ********** Windows Update Installation Files ********** */
    if (IsDlgButtonChecked(hDlg, IDC_WUIF) == BST_CHECKED){
        updateStatus(L"Deleting Windows Update Installation Files...");
        
        wchar *windir = expand(L"%systemroot%");

        StringList *dirs = getFiles(windir, L"$*Uninstall*$", 28, 1);
        StringList *temp = dirs;
        while (temp != NULL){
            wchar *path = (wchar*)calloc(wcslen(windir) + wcslen(temp->string) + 2, sizeof(wchar));
            wsprintf(path, L"%s\\%s", windir, temp->string);
            emptyFolder(path);
            
            logit(L"\r\n");
            logit(path);
            logit(L" <- ");
            if (!RemoveDirectory(path))
                printLastError();
            else
                logit(L"Successfully deleted.");
            
            free(path);
            temp = temp->next;
        }
        freeStringList(dirs);
        
        free(windir);
    }
    
    /* ********** WMI Logs ********** */
    if (IsDlgButtonChecked(hDlg, IDC_WMILOGS) == BST_CHECKED){
        updateStatus(L"Deleting WMI Logs...");
        
        wchar *logs = expand(L"%systemroot%\\system32\\wbem\\Logs");
        StringList *files = getFiles(logs, L"*.lo?", 0, 0);
        StringList *tempFile = files;
        while (tempFile != NULL){
            wchar *path = (wchar*)calloc(wcslen(logs) + wcslen(tempFile->string) + 2, sizeof(wchar));
            wsprintf(path, L"%s\\%s", logs, tempFile->string);
            deleteFile(path, 1);
            free(path);
            tempFile = tempFile->next;
        }
        freeStringList(files);
        free(logs);
    }
    
    /* Search for files matching fileFilter entries. */
    if (fileFilters != NULL){
        updateStatus(L"Scanning for files...");
        
        wchar *root = expand(L"%systemdrive%");
        
        delSearch(root, fileFilters);
        
        free(root);
        freeStringList(fileFilters);
    }
    
    /* Log space cleaned. */
    wchar *space = (wchar*)calloc(45, sizeof(wchar));
    wsprintf(space, L"Total space cleaned: %d bytes", bytes);
    if (logfile != NULL){
        logit(L"\r\n\r\n");
        logit(space);
    }
    else
        MessageBox(hDiag, space, L"PureRa - Done!", MB_ICONINFORMATION);
    
    free(space);
    
    updateStatus(L"Opening log file...");
    finishLog();
    
    updateStatus(L"Finished!");
}

/* About Dialog procedure */
BOOL CALLBACK AboutDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam){
    switch (msg){
        case WM_COMMAND:{
            switch (LOWORD(wParam)){
                case IDC_VISIT:{
                    ShellExecute(hDlg, L"open", L"http://raproducts.org", NULL, NULL, SW_SHOWNORMAL);
                }
                break;
                case IDC_CLOSE:{
                    EndDialog(hDlg, wParam);
                    PostQuitMessage(0);
                    return TRUE;
                }
                break;
            }
            case WM_CLOSE:{
                PostQuitMessage(0);
                return TRUE;
            }
            break;
        }
        break;
        
    }
    
    return FALSE;
}

/* Dialog procedure for first screen. */
BOOL CALLBACK FirstDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam){
    switch (msg){
        case WM_COMMAND:{
            switch (LOWORD(wParam)){
                case IDC_NEXT:{
                    /* Switch pages. */
                    ShowWindow(first, SW_HIDE);
                    ShowWindow(second, SW_SHOW);
                    
                    /* Update status. */
                    updateStatus(L"Select cleaning options.");
                    
                    /* Resize dialog. */
                    RECT rc;
        			GetWindowRect(hDiag, &rc);
        			MoveWindow(hDiag, rc.left, rc.top, (rc.right - rc.left) * 2, rc.bottom - rc.top, TRUE);
        			
                    SetWindowPos(GetDlgItem(hDiag, IDC_DESC), HWND_TOP, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
                }
                break;
                case IDC_ABOUT:{
                    DialogBox(hInst, MAKEINTRESOURCE(IDD_ABOUT), hDiag, AboutDlgProc);
                }
                break;
                case IDC_EXIT:{
                    EndDialog(hDiag, wParam);
                    PostQuitMessage(0);
                    return TRUE;
                }
                break;
            }
        }
        break;
    }
    return FALSE;
}

/* Dialog procedure for second screen. */
BOOL CALLBACK SecondDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam){
    switch (msg){
        case WM_INITDIALOG:{
            CheckDlgButton(hDlg, IDC_CREATELOG, BST_CHECKED);
        }
        break;
        case WM_COMMAND:{
            switch (LOWORD(wParam)){
                case IDC_BACK:{
                    /* Switch pages. */
                    ShowWindow(second, SW_HIDE);
                    ShowWindow(first, SW_SHOW);
                    
                    /* Update status. */
                    updateStatus(L"Please select an option.");
                    
                    /* Resize dialog. */
                    RECT rc;
        			GetWindowRect(hDiag, &rc);
        			MoveWindow(hDiag, rc.left, rc.top, (rc.right - rc.left) / 2, rc.bottom - rc.top, TRUE);
                }
                break;
                case IDC_CHECKALL: {
                    if (HIWORD(wParam)==BN_CLICKED){
						if (IsDlgButtonChecked(hDlg, IDC_CHECKALL)==BST_CHECKED){
							SetDlgItemText(hDlg, IDC_CHECKALL, L"Uncheck All");
							
							SendDlgItemMessage(hDlg, IDC_SQM, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_CONFIGMSI, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_FNTCACHE, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_ICONCACHE, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_TEMP, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_THUMBSDB, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_RECYCLEBIN, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_SPYBOT, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_SOFTDIST, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_WUIF, BM_SETCHECK, BST_CHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_WMILOGS, BM_SETCHECK, BST_CHECKED, 0);
						}
						else {
							SetDlgItemText(hDlg, IDC_CHECKALL, L"Check All");	
							
							SendDlgItemMessage(hDlg, IDC_SQM, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_CONFIGMSI, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_FNTCACHE, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_ICONCACHE, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_TEMP, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_THUMBSDB, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_RECYCLEBIN, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_SPYBOT, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_SOFTDIST, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_WUIF, BM_SETCHECK, BST_UNCHECKED, 0);
							SendDlgItemMessage(hDlg, IDC_WMILOGS, BM_SETCHECK, BST_UNCHECKED, 0);
						}
					}
                }
                break;
                case IDC_CLEAN:{
                    /* Make sure at least one checkbox is checked. */
                    if (IsDlgButtonChecked(hDlg, IDC_SQM) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_CONFIGMSI) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_FNTCACHE) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_ICONCACHE) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_TEMP) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_THUMBSDB) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_RECYCLEBIN) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_SOFTDIST) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_SPYBOT) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_WUIF) != BST_CHECKED &&
                        IsDlgButtonChecked(hDlg, IDC_WMILOGS) != BST_CHECKED)
                        MessageBox(hDiag, L"You must select something to clean!", L"PureRa - Error", MB_ICONERROR);
                    else              
                        CreateThread(NULL, 0, (void*)clean, (void*)hDlg, 0, NULL);
                }
                break;
            }
        }
        break;
    }
    return FALSE;
}

/* Dialog procedure for main PureRa dialog. */
BOOL CALLBACK MainDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam){
    switch (msg){
        case WM_INITDIALOG:{
            hDiag = hDlg;
                      
            /* Load screens. */
            first = CreateDialogIndirect(hInst, LoadResource(hInst, FindResource(NULL, MAKEINTRESOURCE(IDD_FIRST), RT_DIALOG)), hDlg, FirstDlgProc);
            SetWindowPos(first, HWND_TOP, 8, 60, 0, 0, SWP_NOSIZE);
            second = CreateDialogIndirect(hInst, LoadResource(hInst, FindResource(NULL, MAKEINTRESOURCE(IDD_SECOND), RT_DIALOG)), hDlg, SecondDlgProc);
            SetWindowPos(second, HWND_TOP, 8, 0, 0, 0, SWP_NOSIZE);
            
            /* Load banner. */
            HWND banner = CreateDialogIndirect(hInst, LoadResource(hInst, FindResource(NULL, MAKEINTRESOURCE(IDD_BANNER), RT_DIALOG)), hDlg, NULL);
            SetWindowPos(banner, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE);

            /* Load PureRa logo. */
            HBITMAP hBan = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_PURERALOGO));
            SendMessage(GetDlgItem(banner, IDC_LOGO), STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)hBan);
            
            /* Show first screen. */
            ShowWindow(first, SW_SHOW);
        }
        break;
        case WM_CLOSE:{
            PostQuitMessage(0);
            return TRUE;
        }
    }
    return FALSE;
}

/* Program entry point. */
int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int cmdShow){    
    hInst = hThisInstance;
    
    /* Create the PureRa dialog. */
    hDiag = CreateDialog(hInst, MAKEINTRESOURCE(IDD_PURERADIAG), NULL, MainDlgProc);    
                
    /* Message Loop. */        
    MSG msg;
    while (GetMessage(&msg, NULL, 0, 0)){
        if (!IsWindow(hDiag) || !IsDialogMessage(hDiag, &msg)){
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }
    
    return 0;
}
